@@ -351,6 +351,12 @@ impl Gizmo {
         let mut origin = screen_to_world * Vec4::new(x, -y, -1.0, 1.0);
         origin /= origin.w;
         let mut target = screen_to_world * Vec4::new(x, -y, 1.0, 1.0);
+
+        // w is zero when far plane is set to infinity
+        if target.w.abs() < 1e-7 {
+            target.w = 1e-7;
+        }
+
         target /= target.w;
 
         let direction = target.sub(origin).xyz().normalize();
@@ -547,11 +553,11 @@ struct GizmoState {
 
 pub(crate) trait WidgetData: Sized + Default + Copy + Clone + Send + Sync + 'static {
     fn load(ctx: &Context, gizmo_id: Id) -> Self {
-        *ctx.memory().id_data_temp.get_or_default(gizmo_id)
+        *ctx.memory().data.get_temp_mut_or_default(gizmo_id)
     }
 
     fn save(self, ctx: &Context, gizmo_id: Id) {
-        ctx.memory().id_data_temp.insert(gizmo_id, self);
+        ctx.memory().data.insert_temp(gizmo_id, self);
     }
 }
 
